/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 * This class provides a tag (only) implementation of node discovery that will allow the user to hardcode the url into
 * the body of this tag and specify the title in the attributes. That hardcoded information is then read by this tag,
 * combined with the title to make a {@link BreadCrumbNode} and that node is put in a page level attribute called {@link
 * BreadCrumbTrailTag#BREADCRUMB_CURRENT_NODE}.
 * <p/>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class InlineTextNodeDiscoveryTag extends BodyTagSupport {

    /**
     * This field represents the title associated with this node.  This variable is set initially to <tt>null</tt>, is
     * mutable and can be <tt>null</tt> but never an empty string.  A <tt>null</tt> title means that the title for the
     * current node should default that found in the {@link com.topcoder.web.ui.tag.breadcrumbtrail.PathDiscovery} data
     * source. This variable is referenced by the associated getter/setter methods
     */
    private String title = null;

    /**
     * Constructs a new InlineTextNodeDiscoveryTag.
     */
    public InlineTextNodeDiscoveryTag() {
        super();
    }

    /**
     * This method tarts the processing of this tag by telling the support class to evaluate the body.
     *
     * @return Always {@link #EVAL_BODY_BUFFERED}
     */
    public int doStartTag() {
        return EVAL_BODY_BUFFERED;
    }

    /**
     * This method processes the body content. This tag will simply copy the body content, create a {@link
     * BreadCrumbNode} and set it to the attributes as a page level attribute called {@link
     * BreadCrumbTrailTag#BREADCRUMB_CURRENT_NODE} and then clear the body. Please note that the body is never written
     * out as this tag doesn't render anything.
     *
     * @return Always {@link #SKIP_BODY}
     *
     * @throws BreadCrumbException if the body contained nothing
     */
    public int doAfterBody() throws BreadCrumbException {
        final String contentStr = bodyContent.getString();
        if (contentStr == null) {
            throw new BreadCrumbException("The tag body evaluated to a null body string.");
        }
        if (contentStr.trim().length() == 0) {
            throw new BreadCrumbException("The tag body evaluated to an empty body string.");
        }

        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE,
            new BreadCrumbNode(contentStr, getTitle()), PageContext.PAGE_SCOPE);
        return SKIP_BODY;
    }

    /**
     * This method returns the value of the title property.
     *
     * @return The possibly <tt>null</tt>, non-empty string representing the title
     */
    public String getTitle() {
        return title;
    }

    /**
     * This method sets the value of the title property.
     *
     * @param title A possibly <tt>null</tt>, possibly empty (trim'd) string representing the title
     */
    public void setTitle(final String title) {
        if (title != null && title.trim().length() == 0) {
            this.title = null;
        } else {
            this.title = title;
        }
    }
}
