/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.jsp.tagext.TagSupport;
import java.io.IOException;


/**
 * This tag will format the node and style using a template.
 * <p/>
 * A template is a separate file that will be included in the out stream. This class will take the current node/style
 * and define them as request attributes when the template file is included (those attributes can then be used by the
 * template file to format the correct representation of them).
 * <p/>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class TemplateNodeFormatterTag extends TagSupport {

    /**
     * This field represents the template file that will be used when formatting this tag. This variable is set by the
     * {@link #TemplateNodeFormatterTag(String)} constructor and is set to <tt>null</tt> in the {@link
     * #TemplateNodeFormatterTag()} constructor, is mutable and can be <tt>null</tt> or an empty (trim'd) string to
     * specify template is to be used (and thus nothing produced by this tag). Please note that it is expected (but not
     * required) that the value will be a non-<tt>null</tt>, non-empty String. This variable is referenced by the
     * associated getter/setter methods and the doStartTag methods.
     */
    private String template;

    /**
     * Constructs the TemplateNodeFormatterTag using no constructor arg (called by the JSP engine to construct a tag).
     */
    public TemplateNodeFormatterTag() {
        template = null;
    }

    /**
     * Constructs the TemplateNodeFormatterTag using the configuration manager to set the initial value of the
     * template.
     *
     * @param template A possibly <tt>null</tt>, possibly empty (trim'd) string representing the template to use
     */
    public TemplateNodeFormatterTag(final String template) {
        this.template = template;
    }

    /**
     * This method retrieves value of the template variable.
     *
     * @return a possibly <tt>null</tt>, possibly empty (trim'd) string representing the template to use
     */
    public String getTemplate() {
        return template;
    }

    /**
     * This method sets the template that will be used for rendering this tag.
     *
     * @param template a possibly <tt>null</tt>, possibly empty (trim'd) string representing the template to use
     */
    public void setTemplate(final String template) {
        this.template = template;
    }

    /**
     * This method will take the current node and format, make them into request attributes and include the template
     * file (which is assumed to use the request attributes).
     *
     * @return Always {@link #SKIP_BODY}
     *
     * @throws BreadCrumbException in case the page context does not contain the information necessary to render the
     *                             anchor or the rendering failed
     */
    public int doStartTag() throws BreadCrumbException {
        if (template == null || template.trim().length() == 0) {
            return SKIP_BODY;
        }

        //retrieve the node to be rendered
        final BreadCrumbNode breadCrumbNode = (BreadCrumbNode) NodeFormatterUtils.getRequiredPageContextAttribute(
            pageContext, BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, BreadCrumbNode.class);

        // retrieve the style class name to be used
        String styleClass = (String) NodeFormatterUtils.getOptionalPageContextAttribute(
            pageContext, BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, String.class);
        if (styleClass != null && styleClass.trim().length() == 0) {
            styleClass = null;
        }

        //set values as request attributes to be available to the template
        final ServletRequest request = pageContext.getRequest();
        request.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, breadCrumbNode);
        request.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, styleClass);

        //include the template
        try {
            pageContext.include(template);
        } catch (ServletException e) {
            throw new BreadCrumbException("Error while evaluating template.", e);
        } catch (IOException e) {
            throw new BreadCrumbException("Error while evaluating template.", e);
        }

        return SKIP_BODY;
    }

}
