/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

/**
 * This abstract implementation of the {@link NodeMatcher} interface provides title and url type services to subclasses.
 * This implementation will provide a title and url holder variable and a getter methods.
 * <p/>
 * It is strongly encouraged that all implementations of the {@link NodeMatcher} inherit from this abstract base. This
 * abstract base currently, only provides title/url services but may provide additional services in the future.
 * <p/>
 * <b>Thread safety: </b> This class is thread safe by being immutable.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public abstract class AbstractNodeMatcher implements NodeMatcher {

    /**
     * This field represents the default title that a node can take if it matches this node matcher.  This variable is
     * set by the constructor, is immutable and can never be <tt>null</tt> nor an empty (trim'd) string,  The title will
     * be referenced in the getTitle method.
     */
    private final String title;

    /**
     * This field represents the default url that can be used if not contextual information can be used.  This variable
     * is set by the constructor, is immutable and can never be <tt>null</tt> nor an empty (trim'd) string,  The url
     * will be referenced in the getTitle method.
     */
    private final String url;

    /**
     * Constructs the abstract base from the given title and url.
     *
     * @param title A non-<tt>null</tt>, non-empty (trim'd) title to use
     * @param url   A non-<tt>null</tt>, non-empty (trim'd) url to use
     *
     * @throws IllegalArgumentException if either arg is <tt>null</tt> or an empty (trim'd) string
     */
    protected AbstractNodeMatcher(final String title, final String url) {
        if (title == null) {
            throw new IllegalArgumentException("The parameter named [title] was null.");
        }
        if (url == null) {
            throw new IllegalArgumentException("The parameter named [url] was null.");
        }
        if (title.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [title] was an empty String.");
        }
        if (url.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [url] was an empty String.");
        }

        this.title = title;
        this.url = url;
    }

    /**
     * This method returns the default title.
     *
     * @return A non-<tt>null</tt>, non-empty (trim'd) string representing the title
     */
    public String getTitle() {
        return title;
    }

    /**
     * this method returns the url.
     *
     * @return A non-<tt>null</tt>, non-empty (trim'd) string representing the url to use
     */
    public String getUrl() {
        return url;
    }
}
